/*******************************************************************************
        
        copyright:      Copyright (C) Mike Reinhold 2010. All rights reserved
                        
        license:        BSD style: $(LICENSE)
                
        author(s):      Mike Reinhold.
                        
*******************************************************************************/
module tango.util.cipher.pad.ISO10126;

import tango.util.cipher.Cipher;
import tango.math.random.Random;

/** Implements the ISO 10126 padding scheme */
class ISO10126: BlockCipherPadding {
    
    /***************************************************************************
    
            name
            
            Returns:
            The name of the padding scheme          

    ***************************************************************************/
    char[] name() {
        return "ISO 10126";
    }
    
    /***************************************************************************
    
            allowsReordering
            
            Returns:
            does the padding allow safe reordering?
            
            Remarks:
            See the classes in tango.util.cipher.order for further details. 

    ***************************************************************************/
    bool allowsReordering(){
        return true;
    }

    /***************************************************************************
    
            pad
            
            Params:
            len = number of pad bytes to be generated
            
            Returns:
            a ubyte array containing the padding bytes

    ***************************************************************************/
    ubyte[] pad(uint len) {
        ubyte[] pad = new ubyte[len];

        // set pad bytes to random values
        for(int i = 0; i < pad.length - 1; i++) {
            pad[i] = rand.uniform!(ubyte)();
        }

        // set last pad byte to the number of pad bytes
        if(len > 0)
            pad[len - 1] = cast(ubyte) len;

        return pad;
    }

    /***************************************************************************
    
            unpad
            
            Params:
            input_ = the padded text
            
            Returns:
            the number of pad bytes.

    ***************************************************************************/    
    uint unpad(void[] input_) {
        auto input = cast(ubyte[]) input_;
        return cast(uint) (input[input.length - 1]); // the number of pad bytes
    }
}